#include "contiki.h"
#include "lib/random.h"
#include "net/rime/rime.h"
#include "dev/serial-line.h"

#include <stdio.h>
#include <stdlib.h>

linkaddr_t mypeer;
process_event_t END_OF_BCAST;

struct broadcast_message {
  uint8_t seqno;
};


static struct broadcast_conn broadcast;
static struct unicast_conn unicast;

PROCESS(broadcast_process, "Broadcast process");
PROCESS(unicast_process, "Unicast process");
AUTOSTART_PROCESSES(&unicast_process);

static void broadcast_recv(struct broadcast_conn *c, const linkaddr_t *from){
    
  struct broadcast_message *m;
  m = packetbuf_dataptr();
  mypeer = *from;
  printf ("%d RCV BCAST seq %d from %d,%d u16 %d\n", linkaddr_node_addr.u16, m->seqno, mypeer.u8[0], mypeer.u8[1], mypeer.u16);  
}
static const struct broadcast_callbacks broadcast_call = {broadcast_recv};

static void recv_uc(struct unicast_conn *c, const linkaddr_t *from){

  uint16_t * msg = packetbuf_dataptr(); 
  uint16_t seq = msg[0];
  printf("%d %d\n ", seq, packetbuf_datalen());
}

static const struct unicast_callbacks unicast_callbacks = {recv_uc};

PROCESS_THREAD(broadcast_process, ev, data){
  
  PROCESS_EXITHANDLER(broadcast_close(&broadcast);)
  PROCESS_BEGIN();

  static struct etimer bt; //time between two braodcast
  static struct timer bt_phase; // broadcast phase duration
  struct broadcast_message msg;
  static uint8_t seqno=0;

  broadcast_open(&broadcast, 129, &broadcast_call);

  timer_set(&bt_phase, 10*CLOCK_SECOND );
  
  while (!timer_expired(&bt_phase)){
    
    /* Send a broadcast every 2 - 4 seconds */
    
    etimer_set(&bt, CLOCK_SECOND * 2 + random_rand() % (CLOCK_SECOND * 2));
    PROCESS_WAIT_EVENT_UNTIL(etimer_expired(&bt));

    msg.seqno = seqno;
    packetbuf_copyfrom(&msg, sizeof(struct broadcast_message));
    broadcast_send(&broadcast);
    seqno++;
  }
  
  process_post_synch (&unicast_process, END_OF_BCAST, &seqno);
  
  PROCESS_EXIT();
  PROCESS_END();
}


PROCESS_THREAD(unicast_process, ev, data){
  PROCESS_EXITHANDLER(unicast_close(&unicast));
  PROCESS_BEGIN();

  process_start(&broadcast_process, NULL);
  PROCESS_WAIT_EVENT_UNTIL(ev == END_OF_BCAST);
  //printf("END OF BROADCAST OK OK\n\n");
  
  unicast_open(&unicast, 146, &unicast_callbacks);
    
 
  if (linkaddr_node_addr.u16 < mypeer.u16)
    printf ("READY TO SEND ...\n");
  
  uint16_t seqno;
  while(1) {
    
    PROCESS_YIELD();
    
    if (linkaddr_node_addr.u16 < mypeer.u16){
            
      uint8_t size;
      if(ev == serial_line_event_message) {

	size = atoi(strtok(data, " "));//atoi(data);
	data = strtok(NULL, " ");
	seqno = atoi(strtok(data, " "));
	printf("received line: %d seqno %d\n",size, seqno);
	 
	packetbuf_clear();
	uint16_t * msg = packetbuf_dataptr();
	msg[0] = seqno; //msg[0] = ++seq;
	packetbuf_copyfrom(msg, size+2);
	
	printf("sending unicast size %d to %d with seq %d msg  %d\n", packetbuf_datalen(), mypeer.u16, seqno, msg[0]);
	

	unicast_send(&unicast, &mypeer);
      }//if serial
    }//if sender
  }//while
   
  PROCESS_EXIT();
  PROCESS_END();
}
  
